@@ -12,7 +12,7 @@
 #include <assert.h>
 
 #include "librhash/rhash.h"
-#include "librhash/timing.h"
+#include "librhash/rhash_timing.h"
 #include "parse_cmdline.h"
 #include "rhash_main.h"
 #include "hash_print.h"
@@ -59,6 +59,7 @@ static void re_init_rhash_context(struct file_info *info)
 {
 	if(rhash_data.rctx != 0) {
 		if(opt.mode & (MODE_CHECK | MODE_CHECK_EMBEDDED)) {
+			/* a set of hash sums can change from file to file */
 			rhash_free(rhash_data.rctx);
 			rhash_data.rctx = 0;
 		} else {
@@ -67,16 +68,18 @@ static void re_init_rhash_context(struct file_info *info)
 				rhash_reset(rhash_data.rctx);
 			} else {
 				/* add another file to the torrent batch */
-				rhash_transmit(RMSG_BT_ADD_FILE, info->rctx,
+				rhash_transmit(RMSG_BT_ADD_FILE, rhash_data.rctx,
 					RHASH_STR2UPTR((char*)file_info_get_utf8_print_path(info)), (rhash_uptr_t)&info->size);
+				return;
 			}
-			return;
 		}
 	}
 
-	info->rctx = rhash_data.rctx = rhash_init(info->sums_flags);
+	if(rhash_data.rctx == 0) {
+		info->rctx = rhash_data.rctx = rhash_init(info->sums_flags);
+	}
 
-	/* initialize BitTorrent data */
+	/* re-initialize BitTorrent data */
 	if(info->sums_flags & RHASH_BTIH) {
 		init_btih_data(info);
 	}
@@ -93,9 +96,9 @@ static int calc_sums(struct file_info *info)
 {
 	FILE* fd = stdin; /* stdin */
 	int res;
-	uint64_t initial_size;
 
-	if(IS_DASH_STR(info->full_path)) {
+	assert(info->file);
+	if(info->file->mode & FILE_IFSTDIN) {
 		info->print_path = "(stdin)";
 
 #ifdef _WIN32
@@ -105,23 +108,17 @@ static int calc_sums(struct file_info *info)
 		}
 #endif
 	} else {
-		struct rsh_stat_struct stat_buf;
-		/* skip non-existing files */
-		if(rsh_stat(info->full_path, &stat_buf) < 0) {
-			return -1;
-		}
-
-		if((opt.mode & (MODE_CHECK | MODE_CHECK_EMBEDDED)) && S_ISDIR(stat_buf.st_mode)) {
+		if((opt.mode & (MODE_CHECK | MODE_CHECK_EMBEDDED)) && FILE_ISDIR(info->file)) {
 			errno = EISDIR;
 			return -1;
 		}
 
-		info->size = stat_buf.st_size; /* total size, in bytes */
-		IF_WINDOWS(win32_set_filesize64(info->full_path, &info->size)); /* set correct filesize for large files under win32 */
+		info->size = info->file->size; /* total size, in bytes */
 
 		if(!info->sums_flags) return 0;
 
-		/* skip files opened with exclusive rights without reporting an error */
+		/* skip without reporting an error the files
+		 * opened exclusively by another process */
 		fd = rsh_fopen_bin(info->full_path, "rb");
 		if(!fd) {
 			return -1;
@@ -129,7 +126,8 @@ static int calc_sums(struct file_info *info)
 	}
 
 	re_init_rhash_context(info);
-	initial_size = info->rctx->msg_size;
+	/* save initial msg_size, for correct calculation of percents */
+	info->msg_offset = info->rctx->msg_size;
 
 	if(percents_output->update != 0) {
 		rhash_set_callback(info->rctx, (rhash_callback_t)percents_output->update, info);
@@ -141,7 +139,8 @@ static int calc_sums(struct file_info *info)
 			rhash_final(info->rctx, 0); /* finalize hashing */
 		}
 	}
-	info->size = info->rctx->msg_size - initial_size;
+	/* calculate real file size */
+	info->size = info->rctx->msg_size - info->msg_offset;
 	rhash_data.total_size += info->size;
 
 	if(fd != stdin) fclose(fd);
@@ -235,13 +234,13 @@ static int find_embedded_crc32(const char* filepath, unsigned* crc32_be)
 }
 
 /**
- * Rename given file inserting its crc32 sum enclosed in braces just before
- * the file extension.
+ * Rename given file inserting its crc32 sum enclosed into square braces
+ * and placing it right before the file extension.
  *
  * @param info pointer to the data of the file to rename.
  * @return 0 on success, -1 on fail with error code in errno
  */
-int rename_file_to_embed_crc32(struct file_info *info)
+int rename_file_by_embeding_crc32(struct file_info *info)
 {
 	size_t len = strlen(info->full_path);
 	const char* p = info->full_path + len;
@@ -370,15 +369,15 @@ int calculate_and_print_sums(FILE* out, file_t* file, const char *print_path)
 	int res = 0;
 
 	memset(&info, 0, sizeof(info));
+	info.file = file;
 	info.full_path = rsh_strdup(file->path);
 	file_info_set_print_path(&info, print_path);
 	info.size = 0;
 
 	info.sums_flags = opt.sum_flags;
 
-	if(IS_DASH_STR(info.full_path)) {
+	if(file->mode & FILE_IFSTDIN) {
 		print_path = "(stdin)";
-		memset(&info.stat_buf, 0, sizeof(info.stat_buf));
 	} else {
 		if(file->mode & FILE_IFDIR) return 0; /* don't handle directories */
 		info.size = file->size; /* total size, in bytes */
@@ -396,31 +395,32 @@ int calculate_and_print_sums(FILE* out, file_t* file, const char *print_path)
 			log_file_error(file->path);
 			res = -1;
 		}
+		if(rhash_data.interrupted) {
+			report_interrupted();
+			return 0;
+		}
 	}
 
 	info.time = rhash_timer_stop(&timer);
 	finish_percents(&info, res);
 
+	if(opt.flags & OPT_EMBED_CRC) {
+		/* rename the file */
+		rename_file_by_embeding_crc32(&info);
+	}
+
 	if((opt.mode & MODE_TORRENT) && !opt.bt_batch_file) {
 		save_torrent(&info);
 	}
 
-	if(opt.flags & OPT_EMBED_CRC) {
-		/* rename the file */
-		rename_file_to_embed_crc32(&info);
-	}
-
 	if((opt.mode & MODE_UPDATE) && opt.fmt == FMT_SFV) {
-		file_t file;
-		file.path = info.full_path;
-		rsh_file_stat2(&file, 0);
-
-		print_sfv_header_line(rhash_data.upd_fd, &file, info.full_path);
+		/* updating SFV file: print SFV header line */
+		print_sfv_header_line(rhash_data.upd_fd, file, 0);
 		if(opt.flags & OPT_VERBOSE) {
-			print_sfv_header_line(rhash_data.log, &file, info.full_path);
+			print_sfv_header_line(rhash_data.log, file, 0);
 			fflush(rhash_data.log);
 		}
-		rsh_file_cleanup(&file);
+		rsh_file_cleanup(file);
 	}
 
 	if(rhash_data.print_list && res >= 0) {
@@ -466,6 +466,11 @@ static int verify_sums(struct file_info *info)
 	}
 	info->time = rhash_timer_stop(&timer);
 
+	if(rhash_data.interrupted) {
+		report_interrupted();
+		return 0;
+	}
+
 	if((opt.flags & OPT_EMBED_CRC) && find_embedded_crc32(
 		info->print_path, &info->hc.embedded_crc32_be)) {
 			info->hc.flags |= HC_HAS_EMBCRC32;
@@ -511,6 +516,7 @@ int check_hash_file(file_t* file, int chdir)
 			/* initialize file_info structure */
 			memset(&info, 0, sizeof(info));
 			info.full_path = rsh_strdup(hash_file_path);
+			info.file = file;
 			file_info_set_print_path(&info, info.full_path);
 			info.sums_flags = info.hc.hash_mask = RHASH_CRC32;
 			info.hc.flags = HC_HAS_EMBCRC32;
@@ -518,10 +524,11 @@ int check_hash_file(file_t* file, int chdir)
 
 			res = verify_sums(&info);
 			fflush(rhash_data.out);
-
-			if(res == 0) rhash_data.ok++;
-			else if(res == -1 && errno == ENOENT) rhash_data.miss++;
-			rhash_data.processed++;
+			if(!rhash_data.interrupted) {
+				if(res == 0) rhash_data.ok++;
+				else if(res == -1 && errno == ENOENT) rhash_data.miss++;
+				rhash_data.processed++;
+			}
 
 			free(info.full_path);
 			file_info_destroy(&info);
@@ -536,7 +543,7 @@ int check_hash_file(file_t* file, int chdir)
 	rhash_data.processed = rhash_data.ok = rhash_data.miss = 0;
 	rhash_data.total_size = 0;
 
-	if( IS_DASH_STR(hash_file_path) ) {
+	if(file->mode & FILE_IFSTDIN) {
 		fd = stdin;
 		hash_file_path = "<stdin>";
 	} else if( !(fd = rsh_fopen_bin(hash_file_path, "rb") )) {
@@ -578,7 +585,6 @@ int check_hash_file(file_t* file, int chdir)
 		if(IS_COMMENT(*line) || *line == '\r' || *line == '\n') continue;
 
 		memset(&info, 0, sizeof(info));
-		rhash_data.processed++;
 
 		if(!hash_check_parse_line(line, &info.hc, !feof(fd))) continue;
 		if(info.hc.hash_mask == 0) continue;
@@ -599,28 +605,40 @@ int check_hash_file(file_t* file, int chdir)
 		}
 
 		if(info.print_path != NULL) {
+			file_t file_to_check;
 			int is_absolute = IS_PATH_SEPARATOR(info.print_path[0]);
 			IF_WINDOWS(is_absolute = is_absolute || (info.print_path[0] && info.print_path[1] == ':'));
 
-			/* if filename shall be prepent by a directory path */
+			/* if filename shall be prepended by a directory path */
 			if(pos && !is_absolute) {
 				size_t len = strlen(info.print_path);
-				info.full_path = (char*)rsh_malloc(pos+len+1);
+				info.full_path = (char*)rsh_malloc(pos + len + 1);
 				memcpy(info.full_path, hash_file_path, pos);
-				strcpy(info.full_path+pos, info.print_path);
+				strcpy(info.full_path + pos, info.print_path);
 			} else {
 				info.full_path = rsh_strdup(info.print_path);
 			}
+			memset(&file_to_check, 0, sizeof(file_t));
+			file_to_check.path = info.full_path;
+			rsh_file_stat(&file_to_check);
+			info.file = &file_to_check;
 
 			/* verify hash sums of the file */
 			res = verify_sums(&info);
+
 			fflush(rhash_data.out);
-			free(info.full_path);
+			rsh_file_cleanup(&file_to_check);
 			file_info_destroy(&info);
 
+			if(rhash_data.interrupted) {
+				free(path_without_ext);
+				break;
+			}
+
 			/* update statistics */
 			if(res == 0) rhash_data.ok++;
 			else if(res == -1 && errno == ENOENT) rhash_data.miss++;
+			rhash_data.processed++;
 		}
 		free(path_without_ext);
 	}
@@ -642,9 +660,9 @@ int check_hash_file(file_t* file, int chdir)
 }
 
 /**
- * Print a file info line in SFV header format.
+ * Format file information into SFV line and print it to the specified stream.
  *
- * @param out a stream to print info to
+ * @param out the stream to print the file information to
  * @param file the file info to print
  * @return 0 on success, -1 on fail with error code stored in errno
  */
@@ -652,8 +670,8 @@ int print_sfv_header_line(FILE* out, file_t* file, const char* printpath)
 {
 	char buf[24];
 
-	if(!printpath) printpath = file->path;
-	if(printpath[0] == '.' && IS_PATH_SEPARATOR(printpath[1])) printpath += 2;
+	/* skip stdin stream */
+	if ((file->mode & FILE_IFSTDIN) != 0) return 0;
 
 #ifdef _WIN32
 	/* skip file if it can't be opened with exclusive sharing rights */
@@ -662,6 +680,9 @@ int print_sfv_header_line(FILE* out, file_t* file, const char* printpath)
 	}
 #endif
 
+	if(!printpath) printpath = file->path;
+	if(printpath[0] == '.' && IS_PATH_SEPARATOR(printpath[1])) printpath += 2;
+
 	sprintI64(buf, file->size, 12);
 	fprintf(out, "; %s  ", buf);
 	print_time64(out, file->mtime);
