@@ -87,7 +87,7 @@ static char* print_hex_byte(char *dst, const unsigned char byte, int upper_case)
 /**
  * URL-encode a string.
  *
- * @param dst buffer to receive result or NULL to calculate 
+ * @param dst buffer to receive result or NULL to calculate
  *    the lengths of encoded string
  * @param filename the file name
  * @return the length of the result string
@@ -140,8 +140,8 @@ char* str_tolower(const char* str)
 char* str_trim(char* str)
 {
 	char* last = str + strlen(str) - 1;
-	while(isspace(*str)) str++;
-	while(isspace(*last) && last > str) *(last--) = 0;
+	while(isspace((unsigned char)*str)) str++;
+	while(isspace((unsigned char)*last) && last > str) *(last--) = 0;
 	return str;
 }
 
@@ -283,14 +283,42 @@ char* make_path(const char* dir_path, const char* filename)
 	strcpy(buf, dir_path);
 
 	/* separate directory from filename */
-	if(len > 0 && !IS_PATH_SEPARATOR(buf[len-1]))
+	if(len > 0 && !IS_PATH_SEPARATOR(buf[len-1])) {
 		buf[len++] = SYS_PATH_SEPARATOR;
+	}
 
 	/* append filename */
 	strcpy(buf+len, filename);
 	return buf;
 }
 
+#define IS_ANY_SLASH(c) ((c) == RSH_T('/') || (c) == RSH_T('\\'))
+
+/**
+ * Compare paths.
+ *
+ * @param a the first path
+ * @param b the second path
+ */
+int are_paths_equal(const rsh_tchar* a, const rsh_tchar* b)
+{
+	if (!a || !b) return 0;
+	if (a[0] == RSH_T('.') && IS_ANY_SLASH(a[1])) a += 2;
+	if (b[0] == RSH_T('.') && IS_ANY_SLASH(b[1])) b += 2;
+	
+	for (; *a; ++a, ++b)
+	{
+		if (*a != *b && (!IS_ANY_SLASH(*b) || !IS_ANY_SLASH(*a)))
+		{
+			/* paths are different */
+			return 0;
+		}
+		
+	}
+	/* check if both paths terminated */
+	return (*a == *b);
+}
+
 /**
  * Print time formated as hh:mm.ss YYYY-MM-DD to a file stream.
  *
@@ -334,6 +362,34 @@ unsigned rhash_get_ticks(void)
 #endif
 }
 
+#ifdef _WIN32
+/**
+ * Fill file information in the file_t structure.
+ *
+ * @param file the file information
+ * @return 0 on success, -1 on error
+ */
+int rsh_file_statw(file_t* file)
+{
+	WIN32_FILE_ATTRIBUTE_DATA data;
+
+	/* read file attributes */
+	if(GetFileAttributesExW(file->wpath, GetFileExInfoStandard, &data)) {
+		uint64_t u;
+		file->size  = (((uint64_t)data.nFileSizeHigh) << 32) + data.nFileSizeLow;
+		file->mode = (data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ? FILE_IFDIR : 0);
+
+		/* the number of 100-nanosecond intervals since January 1, 1601 */
+		u = (((uint64_t)data.ftLastWriteTime.dwHighDateTime) << 32) + data.ftLastWriteTime.dwLowDateTime;
+		/* convert to second and subtract the epoch difference */
+		file->mtime = u / 10000000 - 11644473600LL;
+		return 0;
+	}
+	set_errno_from_last_file_error();
+	return -1;
+}
+#endif
+
 /**
  * Fill file information in the file_t structure.
  *
@@ -343,43 +399,41 @@ unsigned rhash_get_ticks(void)
  */
 int rsh_file_stat2(file_t* file, int use_lstat)
 {
-	struct rsh_stat_struct st;
-	int res = -1;
-
 #ifdef _WIN32
 	int i;
 	(void)use_lstat; /* ignore on windows */
 
+	file->mtime = 0;
 	if(file->wpath) {
 		free(file->wpath);
 		file->wpath = NULL;
 	}
 
 	for(i = 0; i < 2; i++) {
-		wchar_t* wpath = c2w(file->path, i);
-		if(wpath == NULL) continue;
-		res = clib_wstat(wpath, &st);
-		if(res == 0) {
-			file->wpath = wpath;
-			file->size  = st.st_size;
+		file->wpath = c2w(file->path, i);
+		if(file->wpath == NULL) continue;
 
-			/* set correct filesize for large files under win32 */
-			win32_set_filesize64(file->path, &file->size);
-			break;
-		}
-		free(wpath);
+		/* return on success */
+		if (rsh_file_statw(file) == 0) return 0;
+
+		free(file->wpath);
+		file->wpath = NULL;
 	}
+	/* NB: usually errno is set by the rsh_file_statw() call */
+	if (!errno) errno = EINVAL;
+	return -1;
 #else
-	res = (use_lstat ? lstat(file->path, &st) :
-		rsh_stat(file->path, &st));
+	struct rsh_stat_struct st;
+	int res = -1;
+	res = (use_lstat ? lstat(file->path, &st) : rsh_stat(file->path, &st));
 	file->size  = st.st_size;
-#endif /* _WIN32 */
-
 	file->mtime = st.st_mtime;
+
 	file->mode  = 0;
 	if(S_ISDIR(st.st_mode)) file->mode |= FILE_IFDIR;
 
 	return res;
+#endif /* _WIN32 */
 }
 
 /**
@@ -395,13 +449,16 @@ int rsh_file_stat(file_t* file)
 
 void rsh_file_cleanup(file_t* file)
 {
-	(void)file;
+	free(file->path);
+	file->path = NULL;
+
 #ifdef _WIN32
-	if(file->wpath) {
-		free(file->wpath);
-		file->wpath = NULL;
-	}
+	free(file->wpath);
+	file->wpath = NULL;
 #endif /* _WIN32 */
+
+	file->mtime = file->size = 0;
+	file->mode = 0;
 }
 
 /* program exit and error reporting functions */
@@ -493,6 +550,7 @@ char* rhash_strdup(const char* str, const char* srcfile, int srcline)
 	return res;
 }
 
+#ifdef _WIN32
 /**
  * Duplicate wide string with reporting memory error to stderr.
  *
@@ -501,21 +559,17 @@ char* rhash_strdup(const char* str, const char* srcfile, int srcline)
  * @param srcline source code line to be reported on fail
  * @return allocated memory buffer with copied string
  */
-wchar_t* rhash_wcsdup(const wchar_t* str, const char* srcfile, int srcline)
+wchar_t* rhash_wcsdup(wchar_t* str, const char* srcfile, int srcline)
 {
-#ifndef __STRICT_ANSI__
 	wchar_t* res = wcsdup(str);
-#else
-	wchar_t* res = (wchar_t*)malloc((wcslen(str) + 1) * sizeof(wchar_t));
-	if(res) wcscpy(res, str);
-#endif
 
 	if(!res) {
-		rsh_report_error(srcfile, srcline, "wcsdup(\"%u\") failed\n", (wcslen(str) + 1));
+		rsh_report_error(srcfile, srcline, "wcsdup() failed\n");
 		rsh_exit(2);
 	}
 	return res;
 }
+#endif
 
 /**
  * Reallocates a buffer via realloc with reporting memory error to stderr.
@@ -601,7 +655,7 @@ void rsh_vector_free(vector_t* vect)
  */
 void rsh_vector_add_ptr(vector_t* vect, void* item)
 {
-	/* check if vect contains enough space for next item */
+	/* check if vect contains enough space for the next item */
 	if(vect->size >= vect->allocated) {
 		size_t size = (vect->allocated==0 ? 128 : vect->allocated * 2);
 		vect->array = (void**)rsh_realloc(vect->array, size * sizeof(void*));
