@@ -103,7 +103,7 @@ unsigned get_ctz(unsigned x)
 		0, 1, 28, 2, 29, 14, 24, 3, 30, 22, 20, 15, 25, 17, 4, 8,
 		31, 27, 13, 23, 21, 19, 16, 7, 26, 12, 18, 6, 11, 5, 10, 9
 	};
-	return (unsigned)bit_pos[((uint32_t)((x & -x) * 0x077CB531U)) >> 27];
+	return bit_pos[((uint32_t)((x & -(int)x) * 0x077CB531U)) >> 27];
 }
 #endif /* (GCC >= 4.3) */
 
@@ -135,7 +135,7 @@ static unsigned hash_check_mask_by_digest_size(int digest_size)
 	static unsigned mask[26] = { 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,
 		0,0,0,0,0,0,0,0, 0,0 };
 	int code;
-	
+
 	if(mask[25] == 0) {
 		unsigned hid;
 		for(hid = 1; hid <= RHASH_ALL_HASHES; hid <<= 1) {
@@ -223,6 +223,16 @@ static unsigned char test_hash_string(char **ptr, char *end, int *p_len)
 	return hash_type;
 }
 
+/**
+ * Detect ASCII-7 white spaces (not including Unicode whitespaces).
+ * Note that isspace() is locale specific and detect Unicode spaces,
+ * like U+00A0.
+ */
+static int rhash_isspace(char ch)
+{
+	return (((unsigned char)ch) <= 0x7F && isspace((unsigned char)ch));
+}
+
 /**
  * Information about found token
  */
@@ -242,7 +252,7 @@ typedef struct hc_search
  * string. The format string can contain the following special characters:
  * '\1' - hash function name, '\2' - any hash, '\3' - specified hash,
  * '\4' - an URL-encoded file name, '\5' - a file size,
- * '\6' - a required-space, '\7' - a space or string end. 
+ * '\6' - a required-space, '\7' - a space or string end.
  * A space ' ' means 0 or more space characters.
  * '$' - parse the rest of the buffer and the format string backward.
  * Other (non-special) symbols mean themselves.
@@ -314,9 +324,9 @@ static int hash_check_find_str(hc_search *search, const char* format)
 		case '\3':
 			if(backward) {
 				hv.format = test_hash_string(&end, begin, &len);
-				hv.offset = end - hc->data;
+				hv.offset = (unsigned short)(end - hc->data);
 			} else {
-				hv.offset = begin - hc->data;
+				hv.offset = (unsigned short)(begin - hc->data);
 				hv.format = test_hash_string(&begin, end, &len);
 			}
 			if(!hv.format) return 0;
@@ -353,8 +363,8 @@ static int hash_check_find_str(hc_search *search, const char* format)
 		case '\6':
 		case '\7':
 		case ' ':
-			if(backward) for(; begin < end && isspace(end[-1]); end--, len++);
-			else for(; isspace(*begin) && begin < end; begin++, len++);
+			if(backward) for(; begin < end && rhash_isspace(end[-1]); end--, len++);
+			else for(; rhash_isspace(*begin) && begin < end; begin++, len++);
 			/* check if space is mandatory */
 			if(*search_str != ' ' && len == 0) {
 				/* for '\6' check (len > 0) */
@@ -415,8 +425,11 @@ int hash_check_parse_line(char* line, hash_check* hashes, int check_eol)
 	if( line[0]=='\0' || (le[-1] != '\n' && check_eol) ) return 0;
 
 	/* note: not using str_tim because 'le' is re-used below */
-	while(isspace(le[-1]) && le > line) *(--le) = 0; /* remove trailing white spaces */
-	while(isspace(*line)) line++;  /* skip white spaces at the start of the line */
+
+	/* remove trailing white spaces */
+	while(rhash_isspace(le[-1]) && le > line) *(--le) = 0;
+	/* skip white spaces at the start of the line */
+	while(rhash_isspace(*line)) line++;
 
 	memset(&hs, 0, sizeof(hs));
 	hs.begin = line;
@@ -621,7 +634,7 @@ int hash_check_verify(hash_check* hashes, struct rhash_context* ctx)
 	int j;
 
 	/* verify file size, if present */
-	if((hashes->flags & HC_HAS_FILESIZE) != 0 && 
+	if((hashes->flags & HC_HAS_FILESIZE) != 0 &&
 		hashes->file_size != ctx->msg_size) {
 		hashes->flags |= HC_WRONG_FILESIZE;
 	}
