@@ -329,7 +329,7 @@ static void fprint_ed2k_url(FILE* out, struct file_info *info, int print_type)
  * @param out the stream to output to
  * @param filesize the 64-bit integer to output, usually a file size
  * @param width minimal width of integer to output
- * @param flag =1 if the integer shall be prepent by zeros
+ * @param flag =1 if the integer shall be prepended by zeros
  */
 static void fprintI64(FILE* out, uint64_t filesize, int width, int zero_pad)
 {
@@ -402,7 +402,7 @@ void print_line(FILE* out, print_item* list, struct file_info *info)
 				fprintf(out, "%s", url);
 				break;
 			case PRINT_MTIME: /* the last-modified tine of the filename */
-				print_time(out, info->stat_buf.st_mtime);
+				print_time64(out, info->file->mtime);
 				break;
 			case PRINT_SIZE: /* file size */
 				fprintI64(out, info->size, list->width, (list->flags & PRINT_FLAG_PAD_WITH_ZERO));
@@ -425,6 +425,7 @@ void free_print_list(print_item* list)
 {
 	while(list) {
 		print_item* next = list->next;
+		free((char*)list->data);
 		free(list);
 		list = next;
 	}
