@@ -8,7 +8,7 @@
 #include <sys/stat.h>
 #include <errno.h>
 
-#include "librhash/timing.h"
+#include "librhash/rhash_timing.h"
 #include "win_utils.h"
 #include "parse_cmdline.h"
 #include "output.h"
@@ -162,16 +162,19 @@ static int add_sums_to_file(const char* hash_file_path, char* dir_path, file_set
 	}
 
 	/* append hash sums to the updated crc file */
-	for(i = 0; i < files_to_add->size; i++, rhash_data.processed++) {
+	for(i = 0; i < files_to_add->size; i++, rhash_data.processed++)
+	{
 		file_t file;
-		char *allocated = 0;
 		char *print_path = file_set_get(files_to_add, i)->filepath;
-		file.path = print_path;
+		file.wpath = 0;
 
 		if(dir_path[0] != '.' || dir_path[1] != 0) {
 			/* prepend the file path by directory path */
-			file.path = allocated = make_path(dir_path, print_path);
+			file.path = make_path(dir_path, print_path);
+		} else {
+			file.path = rsh_strdup(print_path);
 		}
+
 		if(opt.fmt == FMT_SFV) {
 			if(print_banner) {
 				print_sfv_banner(fd);
@@ -184,7 +187,11 @@ static int add_sums_to_file(const char* hash_file_path, char* dir_path, file_set
 		calculate_and_print_sums(fd, &file, print_path);
 
 		rsh_file_cleanup(&file);
-		free(allocated);
+
+		if(rhash_data.interrupted) {
+			fclose(fd);
+			return 0;
+		}
 	}
 	fclose(fd);
 	log_msg(_("Updated: %s\n"), hash_file_path);
@@ -272,7 +279,7 @@ static int add_new_crc_entries(const char* hash_file_path, file_set *crc_entries
 		/* calculate and write crc sums to the file */
 		res = add_sums_to_file(hash_file_path, dir_path, files_to_add);
 
-		if(res == 0 && opt.fmt == FMT_SFV) {
+		if(res == 0 && opt.fmt == FMT_SFV && !rhash_data.interrupted) {
 			/* move SFV header from the end of updated file to its head */
 			res = fix_sfv_header(hash_file_path);
 		}
@@ -349,7 +356,7 @@ static int fix_sfv_header(const char* hash_file_path)
 		unlink(hash_file_path);
 #endif
 		if(rename(tmp_file, hash_file_path) < 0) {
-			log_error(_("can't move %s to %s: %s\n"), 
+			log_error(_("can't move %s to %s: %s\n"),
 				tmp_file, hash_file_path, strerror(errno));
 			err = 1;
 		}
