@@ -2,7 +2,7 @@
  * based on the Russian Standard GOST R 34.11-94.
  * See also RFC 4357.
  *
- * Copyright: 2009 Alexey Kravchenko <rhash.admin@gmail.com>
+ * Copyright: 2009-2012 Aleksey Kravchenko <rhash.admin@gmail.com>
  *
  * Permission is hereby granted,  free of charge,  to any person  obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -10,6 +10,10 @@
  * the rights to  use, copy, modify,  merge, publish, distribute, sublicense,
  * and/or sell copies  of  the Software,  and to permit  persons  to whom the
  * Software is furnished to do so.
+ *
+ * This program  is  distributed  in  the  hope  that it will be useful,  but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.  Use this program  at  your own risk!
  */
 
 #include <string.h>
@@ -23,7 +27,7 @@ extern unsigned rhash_gost_sbox_cryptpro[4][256];
  * Initialize algorithm context before calculaing hash
  * with test parameters set.
  *
- * @param ctx context to initalize
+ * @param ctx context to initialize
  */
 void rhash_gost_init(gost_ctx *ctx)
 {
@@ -33,7 +37,7 @@ void rhash_gost_init(gost_ctx *ctx)
 /**
  * Initialize GOST algorithm context with CryptoPro parameter set.
  *
- * @param ctx context to initalize
+ * @param ctx context to initialize
  */
 void rhash_gost_cryptopro_init(gost_ctx *ctx)
 {
@@ -55,11 +59,11 @@ void rhash_gost_cryptopro_init(gost_ctx *ctx)
 #ifndef USE_GCC_ASM_IA32
 # define GOST_ENCRYPT_ROUND(key1, key2, sbox) \
 	tmp = (key1) + r; \
-	l ^= (sbox)[tmp & 0xff] ^ ((sbox)+256)[(tmp >> 8) & 0xff] ^ \
-		((sbox)+512)[(tmp >> 16) & 0xff] ^ ((sbox)+768)[tmp >> 24]; \
+	l ^= (sbox)[tmp & 0xff] ^ ((sbox) + 256)[(tmp >> 8) & 0xff] ^ \
+		((sbox) + 512)[(tmp >> 16) & 0xff] ^ ((sbox) + 768)[tmp >> 24]; \
 	tmp = (key2) + l; \
-	r ^= (sbox)[tmp & 0xff] ^ ((sbox)+256)[(tmp >> 8) & 0xff] ^ \
-		((sbox)+512)[(tmp >> 16) & 0xff] ^ ((sbox)+768)[tmp >> 24];
+	r ^= (sbox)[tmp & 0xff] ^ ((sbox) + 256)[(tmp >> 8) & 0xff] ^ \
+		((sbox) + 512)[(tmp >> 16) & 0xff] ^ ((sbox) + 768)[tmp >> 24];
 
 /* encrypt a block with the given key */
 # define GOST_ENCRYPT(result, i, key, hash, sbox) \
@@ -152,8 +156,8 @@ static void rhash_gost_block_compress(gost_ctx *ctx, const unsigned* block)
 			"movl %%ebx, %13\n\t"
 			GOST_ENCRYPT_GCC_ASM_X86() /* optimized for x86 Intel Core 2 */
 			"movl %13, %%ebx\n\t"
-			: "=S" (s[i]), "=D" (s[i+1]) /* 0,1: s[i]=esi, s[i+1]=edi */
-			: "d" (sbox), "D" (ctx->hash[i]), "S" (ctx->hash[i+1]), /* 2,3,4: edx=sbox,edi=r,esi=l */
+			: "=S" (s[i]), "=D" (s[i + 1]) /* 0,1: s[i]=esi, s[i + 1]=edi */
+			: "d" (sbox), "D" (ctx->hash[i]), "S" (ctx->hash[i + 1]), /* 2,3,4: edx=sbox,edi=r,esi=l */
 			"m" (key[0]), "m" (key[1]), "m" (key[2]), "m" (key[3]), /* 5, 6, 7, 8 */
 			"m" (key[4]), "m" (key[5]), "m" (key[6]), "m" (key[7]), /* 9,10,11,12 */
 			"m" (w[0])  /* store EBX in w[0], cause it's used for PIC on *BSD. */
@@ -321,10 +325,10 @@ static void rhash_gost_compute_sum_and_hash(gost_ctx * ctx, const unsigned* bloc
 
 	/* compute the 256-bit sum */
 	for(i = 0; i < 8; i++) {
-		const unsigned old = ctx->sum[i];
 		LOAD_BLOCK_LE(i);
 		ctx->sum[i] += block_le[i] + carry;
-		carry = (ctx->sum[i] < old || ctx->sum[i] < block_le[i] ? 1 : 0);
+		carry = (ctx->sum[i] < block_le[i] ? 1 :
+			ctx->sum[i] == block_le[i] ? carry : 0);
 	}
 #endif /* USE_GCC_ASM_IA32 */
 
