@@ -14,6 +14,7 @@
 #include "librhash/rhash.h"
 #include "win_utils.h"
 #include "file_mask.h"
+#include "find_file.h"
 #include "hash_print.h"
 #include "output.h"
 #include "rhash_main.h"
@@ -26,6 +27,11 @@ typedef struct options_t options_t;
 struct options_t conf_opt; /* config file parsed options */
 struct options_t opt;      /* command line options */
 
+static void print_help_line(const char* option, const char* text)
+{
+	fprintf(rhash_data.out, "%s%s", option, text);
+}
+
 /**
  * Print program help.
  */
@@ -37,58 +43,55 @@ static void print_help(void)
 	fprintf(rhash_data.out, _("%s\n"
 		"Usage: %s [OPTION...] [FILE | -]...\n"
 		"       %s --printf=<format string> [FILE | -]...\n\n"), VERSION_STRING, CMD_FILENAME, CMD_FILENAME);
+	fprintf(rhash_data.out, _("Options:\n"));
 
-	fprintf(rhash_data.out, "%s%s%s%s",
-		_("Options:\n"
-		"  -V, --version Print program version and exit.\n"
-		"  -h, --help    Print this help screen.\n"
-		"  -C, --crc32   Calculate CRC32 hash sum.\n"
-		"      --md4     Calculate MD4   hash sum.\n"
-		"  -M, --md5     Calculate MD5   hash sum.\n"
-		"  -H, --sha1    Calculate SHA1  hash sum.\n"
-		"      --sha224, --sha256, --sha384, --sha512 Calculate SHA2 hash sum.\n"
-		"  -T, --tth     Calculate TTH sum.\n"
-		"      --btih    Calculate BitTorrent InfoHash.\n"
-		"  -A, --aich    Calculate AICH hash.\n"
-		"  -E, --ed2k    Calculate eDonkey hash sum.\n"
-		"  -L, --ed2k-link  Calculate and print eDonkey link.\n"
-		"      --tiger   Calculate Tiger hash sum.\n"
-		"  -G, --gost    Calculate GOST R 34.11-94 hash.\n"
-		"      --gost-cryptopro CryptoPro version of the GOST R 34.11-94 hash.\n"
-		"      --ripemd160  Calculate RIPEMD-160 hash.\n"
-		"      --has160  Calculate HAS-160 hash.\n"
-		"      --edonr256, --edonr512  Calculate EDON-R 256/512 hash.\n"
-		"      --snefru128, --snefru256  Calculate SNEFRU-128/256 hash.\n"
-		"  -a, --all     Calculate all supported hashes.\n"),
-		_("  -c, --check   Check hash files specified by command line.\n"
-		"  -u, --update  Update hash files specified by command line.\n"
-		"  -e, --embed-crc  Rename files by inserting crc32 sum into name.\n"
-		"      --check-embedded  Verify files by crc32 sum embedded in their names.\n"
-		"      --list-hashes  List the names of supported hashes, one per line.\n"
-		"  -B, --benchmark  Benchmark selected algorithm.\n"
-		"  -v, --verbose Be verbose.\n"
-		"  -r, --recursive  Process directories recursively.\n"
-		"      --skip-ok Don't print OK messages for successfully verified files.\n"
-		"  -i, --ignore-case  Ignore case of filenames when updating hash files.\n"
-		"      --percents   Show percents, while calculating or checking hashes.\n"
-		"      --speed   Output per-file and total processing speed.\n"
-		"      --maxdepth=<n> Descend at most <n> levels of directories.\n"
-		"  -o, --output=<file> File to output calculation or checking results.\n"
-		"  -l, --log=<file>    File to log errors and verbose information.\n"
-		"      --sfv     Print hash sums, using SFV format (default).\n"
-		"      --bsd     Print hash sums, using BSD-like format.\n"
-		"      --simple  Print hash sums, using simple format.\n"
-		"  -m, --magnet  Print hash sums  as magnet links.\n"
-		"      --torrent Create torrent files.\n"),
+	print_help_line("  -V, --version ", _("Print program version and exit.\n"));
+	print_help_line("  -h, --help    ", _("Print this help screen.\n"));
+	print_help_line("  -C, --crc32   ", _("Calculate CRC32 hash sum.\n"));
+	print_help_line("      --md4     ", _("Calculate MD4   hash sum.\n"));
+	print_help_line("  -M, --md5     ", _("Calculate MD5   hash sum.\n"));
+	print_help_line("  -H, --sha1    ", _("Calculate SHA1  hash sum.\n"));
+	print_help_line("      --sha224, --sha256, --sha384, --sha512 ", _("Calculate SHA2 hash sum.\n"));
+	print_help_line("      --sha3-224, --sha3-256, --sha3-384, --sha3-512 ", _("Calculate SHA3 hash sum.\n"));
+	print_help_line("  -T, --tth     ", _("Calculate TTH sum.\n"));
+	print_help_line("      --btih    ", _("Calculate BitTorrent InfoHash.\n"));
+	print_help_line("  -A, --aich    ", _("Calculate AICH hash.\n"));
+	print_help_line("  -E, --ed2k    ", _("Calculate eDonkey hash sum.\n"));
+	print_help_line("  -L, --ed2k-link  ", _("Calculate and print eDonkey link.\n"));
+	print_help_line("      --tiger   ", _("Calculate Tiger hash sum.\n"));
+	print_help_line("  -G, --gost    ", _("Calculate GOST R 34.11-94 hash.\n"));
+	print_help_line("      --gost-cryptopro ", _("CryptoPro version of the GOST R 34.11-94 hash.\n"));
+	print_help_line("      --ripemd160  ", _("Calculate RIPEMD-160 hash.\n"));
+	print_help_line("      --has160  ", _("Calculate HAS-160 hash.\n"));
+	print_help_line("      --edonr256, --edonr512  ", _("Calculate EDON-R 256/512 hash.\n"));
+	print_help_line("      --snefru128, --snefru256  ", _("Calculate SNEFRU-128/256 hash.\n"));
+	print_help_line("  -a, --all     ", _("Calculate all supported hashes.\n"));
+	print_help_line("  -c, --check   ", _("Check hash files specified by command line.\n"));
+	print_help_line("  -u, --update  ", _("Update hash files specified by command line.\n"));
+	print_help_line("  -e, --embed-crc  ", _("Rename files by inserting crc32 sum into name.\n"));
+	print_help_line("  -k, --check-embedded  ", _("Verify files by crc32 sum embedded in their names.\n"));
+	print_help_line("      --list-hashes  ", _("List the names of supported hashes, one per line.\n"));
+	print_help_line("  -B, --benchmark  ", _("Benchmark selected algorithm.\n"));
+	print_help_line("  -v, --verbose ", _("Be verbose.\n"));
+	print_help_line("  -r, --recursive  ", _("Process directories recursively.\n"));
+	print_help_line("      --skip-ok ", _("Don't print OK messages for successfully verified files.\n"));
+	print_help_line("  -i, --ignore-case  ", _("Ignore case of filenames when updating hash files.\n"));
+	print_help_line("      --percents   ", _("Show percents, while calculating or checking hashes.\n"));
+	print_help_line("      --speed   ", _("Output per-file and total processing speed.\n"));
+	print_help_line("      --maxdepth=<n> ", _("Descend at most <n> levels of directories.\n"));
+	print_help_line("  -o, --output=<file> ", _("File to output calculation or checking results.\n"));
+	print_help_line("  -l, --log=<file>    ", _("File to log errors and verbose information.\n"));
+	print_help_line("      --sfv     ", _("Print hash sums, using SFV format (default).\n"));
+	print_help_line("      --bsd     ", _("Print hash sums, using BSD-like format.\n"));
+	print_help_line("      --simple  ", _("Print hash sums, using simple format.\n"));
+	print_help_line("  -m, --magnet  ", _("Print hash sums  as magnet links.\n"));
+	print_help_line("      --torrent ", _("Create torrent files.\n"));
 #ifdef _WIN32
-		_("      --ansi    Use Windows codepage for output (Windows only).\n"),
-#else
-		"",
+	print_help_line("      --ansi    ", _("Use Windows codepage for output (Windows only).\n"));
 #endif
-		_("      --template=<file> Load a printf-like template from the <file>\n"
-		"  -p, --printf=<format string>  Format and print hash sums.\n"
-		"                See the RHash manual for details.\n")
-		);
+	print_help_line("      --template=<file> ", _("Load a printf-like template from the <file>\n"));
+	print_help_line("  -p, --printf=<format string>  ",
+		_("Format and print hash sums.\n                See the RHash manual for details.\n"));
 	rsh_exit(0);
 }
 
@@ -128,7 +131,7 @@ static void crc_accept(options_t *o, char* accept_string, unsigned type)
  */
 static void openssl_flags(options_t *o, char* openssl_hashes, unsigned type)
 {
-#ifdef USE_OPENSSL
+#if defined(USE_OPENSSL) || defined(OPENSSL_RUNTIME)
 	char *cur, *next;
 	(void)type;
 	o->openssl_mask = 0x80000000; /* turn off using default mask */
@@ -260,7 +263,7 @@ enum option_type_t
 	F_UFLG = 1, /* set a bit flag in a uint32_t field */
 	F_UENC = F_UFLG | F_OUTPUT_OPT, /* an encoding changing option */
 	F_CSTR = 2 | F_NEED_PARAM, /* store parameter as a C string */
-	F_OPTH = 3 | F_NEED_PARAM | F_OUTPUT_OPT,
+	F_TOUT = 3 | F_NEED_PARAM | F_OUTPUT_OPT,
 	F_VFNC = 4, /* just call a function */
 	F_PFNC = 5 | F_NEED_PARAM, /* process option parameter by calling a handler */
 	F_PRNT = 6, /* print a constant C-string and exit */
@@ -274,7 +277,7 @@ cmdline_opt_t cmdline_opt[] =
 {
 	/* program modes */
 	{ F_UFLG, 'c',   0, "check",  &opt.mode, MODE_CHECK },
-	{ F_UFLG,   0,   0, "check-embedded",  &opt.mode, MODE_CHECK_EMBEDDED },
+	{ F_UFLG, 'k',   0, "check-embedded",  &opt.mode, MODE_CHECK_EMBEDDED },
 	{ F_UFLG, 'u',   0, "update", &opt.mode, MODE_UPDATE },
 	{ F_UFLG, 'B',   0, "benchmark", &opt.mode, MODE_BENCHMARK },
 	{ F_UFLG,   0,   0, "torrent", &opt.mode, MODE_TORRENT },
@@ -292,6 +295,10 @@ cmdline_opt_t cmdline_opt[] =
 	{ F_UFLG,   0,   0, "sha256", &opt.sum_flags, RHASH_SHA256 },
 	{ F_UFLG,   0,   0, "sha384", &opt.sum_flags, RHASH_SHA384 },
 	{ F_UFLG,   0,   0, "sha512", &opt.sum_flags, RHASH_SHA512 },
+	{ F_UFLG,   0,   0, "sha3-224", &opt.sum_flags, RHASH_SHA3_224 },
+	{ F_UFLG,   0,   0, "sha3-256", &opt.sum_flags, RHASH_SHA3_256 },
+	{ F_UFLG,   0,   0, "sha3-384", &opt.sum_flags, RHASH_SHA3_384 },
+	{ F_UFLG,   0,   0, "sha3-512", &opt.sum_flags, RHASH_SHA3_512 },
 	{ F_UFLG,   0,   0, "tiger",  &opt.sum_flags, RHASH_TIGER },
 	{ F_UFLG, 'T',   0, "tth",    &opt.sum_flags, RHASH_TTH },
 	{ F_UFLG,   0,   0, "btih",   &opt.sum_flags, RHASH_BTIH },
@@ -329,8 +336,8 @@ cmdline_opt_t cmdline_opt[] =
 	{ F_UFLG, 'e',   0, "embed-crc",  &opt.flags, OPT_EMBED_CRC },
 	{ F_CSTR,   0,   0, "embed-crc-delimiter", &opt.embed_crc_delimiter, 0 },
 	{ F_PFNC,   0,   0, "path-separator", set_path_separator, 0 },
-	{ F_OPTH, 'o',   0, "output", &opt.output, 0 },
-	{ F_OPTH, 'l',   0, "log",    &opt.log,    0 },
+	{ F_TOUT, 'o',   0, "output", &opt.output, 0 },
+	{ F_TOUT, 'l',   0, "log",    &opt.log,    0 },
 	{ F_PFNC, 'q',   0, "accept", crc_accept, 0 },
 	{ F_PFNC, 't',   0, "crc-accept", crc_accept, 1 },
 	{ F_VFNC,   0,   0, "video",  accept_video, 0 },
@@ -391,25 +398,28 @@ typedef struct parsed_option_t
 static void apply_option(options_t *opts, parsed_option_t* option)
 {
 	cmdline_opt_t* o = option->o;
-	char option_type = o->type;
+	unsigned short option_type = o->type;
 	char* value = NULL;
-	
+
 	/* check if option requires a parameter */
 	if(is_param_required(option_type)) {
 		if(!option->parameter) {
 			log_error(_("argument is required for option %s\n"), option->name);
 			rsh_exit(2);
 		}
+
 #ifdef _WIN32
-		/* convert from UTF-16 if not a filepath */
-		if(option_type != F_OPTH) {
+		if(option_type == F_TOUT) {
+			/* leave the value in UTF-16 */
+			value = (char*)rsh_wcsdup((wchar_t*)option->parameter);
+		} else {
+			/* convert from UTF-16 */
 			value = w2c((wchar_t*)option->parameter);
-			rsh_vector_add_ptr(opt.mem, value);
-		} else
-#endif
-		{
-			value = (char*)option->parameter;
 		}
+		rsh_vector_add_ptr(opt.mem, value);
+#else
+		value = (char*)option->parameter;
+#endif
 	}
 
 	/* process option, choosing the method by type */
@@ -419,7 +429,7 @@ static void apply_option(options_t *opts, parsed_option_t* option)
 		*(unsigned*)((char*)opts + ((char*)o->ptr - (char*)&opt)) |= o->param;
 		break;
 	case F_CSTR:
-	case F_OPTH:
+	case F_TOUT:
 		/* save the option parameter */
 		*(char**)((char*)opts + ((char*)o->ptr - (char*)&opt)) = value;
 		break;
@@ -601,7 +611,7 @@ static void parse_long_option(parsed_option_t* option, rsh_tchar ***parg)
 			}
 		}
 		name[i] = '\0';
-		
+
 		name += 2; /* skip  "--" */
 		length -= 2;
 	} else fail = 1;
@@ -681,7 +691,7 @@ static void parse_cmdline_options(struct parsed_cmd_line_t* cmd_line)
 		}
 
 		assert((*parg)[0] == RSH_T('-') && (*parg)[1] != 0);
-		
+
 		if((*parg)[1] == L'-' && (*parg)[2] == 0) {
 			b_opt_end = 1; /* string "--" means end of options */
 			continue;
@@ -706,7 +716,7 @@ static void parse_cmdline_options(struct parsed_cmd_line_t* cmd_line)
 		} else if((*parg)[1] != 0) {
 			/* found '-'<some string> */
 			rsh_tchar* ptr;
-			
+
 			/* parse short options. A string of several characters is interpreted
 			 * as separate short options */
 			for(ptr = *parg + 1; *ptr; ptr++) {
@@ -726,7 +736,7 @@ static void parse_cmdline_options(struct parsed_cmd_line_t* cmd_line)
 				next_opt->buf[0] = '-', next_opt->buf[1] = ch, next_opt->buf[2] = '\0';
 				next_opt->name = next_opt->buf;
 				next_opt->parameter = NULL;
-				
+
 				/* search for the short option */
 				for(t = cmdline_opt; t->type && ch != t->short1 && ch != t->short2; t++);
 				if(!t->type) fail_on_unknow_option(next_opt->buf);
@@ -762,8 +772,8 @@ static void parse_cmdline_options(struct parsed_cmd_line_t* cmd_line)
  */
 static void apply_cmdline_options(struct parsed_cmd_line_t *cmd_line)
 {
-	int count = cmd_line->options.size;
-	int i;
+	size_t count = cmd_line->options.size;
+	size_t i;
 	for(i = 0; i < count; i++) {
 		parsed_option_t* o = (parsed_option_t*)rsh_blocks_vector_get_ptr(
 			&cmd_line->options, i, 16, sizeof(parsed_option_t));
@@ -793,6 +803,7 @@ static void apply_cmdline_options(struct parsed_cmd_line_t *cmd_line)
 	if(opt.embed_crc_delimiter == 0) opt.embed_crc_delimiter = conf_opt.embed_crc_delimiter;
 	if(!opt.path_separator) opt.path_separator = conf_opt.path_separator;
 	if(opt.find_max_depth < 0) opt.find_max_depth = conf_opt.find_max_depth;
+	if (!(opt.flags & OPT_RECURSIVE)) opt.find_max_depth = 0;
 	if(opt.flags & OPT_EMBED_CRC) opt.sum_flags |= RHASH_CRC32;
 	if(opt.openssl_mask == 0) opt.openssl_mask = conf_opt.openssl_mask;
 
@@ -829,6 +840,10 @@ static void set_default_sums_flags(const char* progName)
 	if(strstr(buf, "sha512")) res |= RHASH_SHA512;
 	if(strstr(buf, "sha224")) res |= RHASH_SHA224;
 	if(strstr(buf, "sha384")) res |= RHASH_SHA384;
+	if(strstr(buf, "sha3-256")) res |= RHASH_SHA3_256;
+	if(strstr(buf, "sha3-512")) res |= RHASH_SHA3_512;
+	if(strstr(buf, "sha3-224")) res |= RHASH_SHA3_224;
+	if(strstr(buf, "sha3-384")) res |= RHASH_SHA3_384;
 	if(strstr(buf, "tiger")) res |= RHASH_TIGER;
 	if(strstr(buf, "tth"))   res |= RHASH_TTH;
 	if(strstr(buf, "btih"))  res |= RHASH_BTIH;
@@ -865,8 +880,8 @@ void options_destroy(struct options_t* o)
 {
 	file_mask_free(o->files_accept);
 	file_mask_free(o->crc_accept);
-	rsh_vector_free(o->cmd_vec);
 	rsh_vector_free(o->mem);
+	destroy_file_search_data(o->search_data);
 }
 
 /**
@@ -908,10 +923,6 @@ static void make_final_options_checks(void)
 void read_options(int argc, char *argv[])
 {
 	struct parsed_cmd_line_t cmd_line;
-#ifdef _WIN32
-	int i;
-	vector_t *expanded_cnames;
-#endif
 
 	memset(&opt, 0, sizeof(opt));
 	opt.mem = rsh_vector_new_simple();
@@ -926,44 +937,30 @@ void read_options(int argc, char *argv[])
 	/* parse command line and apply encoding options */
 	parse_cmdline_options(&cmd_line);
 	read_config();
-	
+
 #ifdef _WIN32
 	/* set default encoding if no encoding options were specified, */
 	/* this should be done here, even if config file was not found. */
 	if( (opt.flags & OPT_ENCODING) == 0 ) opt.flags |= OPT_UTF8;
 #endif
 
-	/* note: encoding and -o/-l options are already applied */
+	/* setup the program output */
 	IF_WINDOWS(setup_console());
-	setup_output(); /* setup program output */
+	setup_output();
 
 	apply_cmdline_options(&cmd_line); /* process the rest of command options */
 
 	/* options were processed, so we don't need them anymore */
 	rsh_blocks_vector_destroy(&cmd_line.options);
-	
-#ifdef _WIN32
-	expanded_cnames = rsh_vector_new_simple();
 
-	/* convert paths to internal encoding and expand wildcards. */
-	for(i = 0; i < cmd_line.n_files; i++) {
-		wchar_t* path = cmd_line.files[i];
-		wchar_t* p = wcschr(path, L'\0') - 1;
-
-		/* strip trailing '\','/' symbols (if not preceded by ':') */
-		for(; p > path && IS_PATH_SEPARATOR_W(*p) && p[-1] != L':'; p--) *p = 0;
-		expand_wildcards(expanded_cnames, path);
-	}
-
-	opt.cmd_vec = expanded_cnames;
-	opt.files = (char**)expanded_cnames->array;
-	opt.n_files = (int)expanded_cnames->size;
-	free(cmd_line.files);
-	LocalFree(cmd_line.warg);
-#else
-	opt.files = cmd_line.files;
+	/* set the files and directories to be processed later */
+	opt.search_data = create_file_search_data(cmd_line.files, cmd_line.n_files, opt.find_max_depth);
 	opt.n_files = cmd_line.n_files;
-	rsh_vector_add_ptr(opt.mem, opt.files);
+
+	free(cmd_line.files);
+
+#ifdef _WIN32
+	LocalFree(cmd_line.warg);
 #endif
 
 	make_final_options_checks();
