@@ -8,6 +8,7 @@
 #include <io.h> /* for isatty */
 #include <assert.h>
 #include <errno.h>
+#include <locale.h>
 
 #include "common_func.h"
 #include "parse_cmdline.h"
@@ -78,7 +79,10 @@ char* wchar_to_cstr(const wchar_t* wstr, int codepage, int* failed)
 	lpUsedDefaultChar = (failed && codepage != CP_UTF8 ? &bUsedDefChar : NULL);
 
 	size = WideCharToMultiByte(codepage, 0, wstr, -1, 0, 0, 0, 0);
-	if(size == 0) return NULL; /* conversion failed */
+	if(size == 0) {
+		if (failed) *failed = 1;
+		return NULL; /* conversion failed */
+	}
 	buf = (char*)rsh_malloc(size);
 	WideCharToMultiByte(codepage, 0, wstr, -1, buf, size, 0, lpUsedDefaultChar);
 	if(failed) *failed = (lpUsedDefaultChar && *lpUsedDefaultChar);
@@ -165,41 +169,6 @@ int win_stat(const char* path, struct rsh_stat_struct *buffer)
 	return res;
 }
 
-/**
- * Get 64-bit file size and store it into given 64-bit integer.
- *
- * @param path file path
- * @param pSize pointer to a 64-bit integer to store file size into
- */
-void win32_set_filesize64(const char* path, uint64_t *pSize)
-{
-	int try_no;
-	for(try_no = 0; try_no < 2; try_no++) {
-		HANDLE hFile;
-		DWORD fileSizeLow, fileSizeHigh;
-
-		wchar_t* wpath = c2w(path, try_no);
-		if(wpath == NULL) continue;
-		hFile = CreateFileW(wpath, 0, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-		free(wpath);
-		if(hFile == INVALID_HANDLE_VALUE) {
-			UINT error = GetLastError();
-			if(error != ERROR_FILE_NOT_FOUND && error != ERROR_PATH_NOT_FOUND) break;
-		} else {
-			fileSizeLow = GetFileSize(hFile, &fileSizeHigh);
-
-			/* it is strange but the correct method to check for error */
-			if(fileSizeLow == 0xffffffff && GetLastError() != 0) {
-				/* printf("error code = %u: %s\n", GetLastError(), filepath); */
-			} else {
-				*pSize = fileSizeLow + ( (uint64_t)fileSizeHigh << 32 );
-			}
-			CloseHandle(hFile);
-			break;
-		}
-	}
-}
-
 /**
  * Check if given file can be opened with exclusive write access.
  *
@@ -223,6 +192,80 @@ int can_open_exclusive(const char* path)
 	return res;
 }
 
+/* the range of error codes for access errors */
+#define MIN_EACCES_RANGE ERROR_WRITE_PROTECT
+#define MAX_EACCES_RANGE ERROR_SHARING_BUFFER_EXCEEDED
+
+/**
+ * Convert the GetLastError() value to the assignable to errno.
+ *
+ * @return errno-compatible error code
+ */
+static int convert_last_error_to_errno(void)
+{
+	DWORD error_code = GetLastError();
+	switch (error_code)
+	{
+	case NO_ERROR:
+		return 0;
+	case ERROR_FILE_NOT_FOUND:
+	case ERROR_PATH_NOT_FOUND:
+	case ERROR_INVALID_DRIVE:
+	case ERROR_BAD_NETPATH:
+	case ERROR_BAD_PATHNAME:
+	case ERROR_FILENAME_EXCED_RANGE:
+		return ENOENT;
+	case ERROR_TOO_MANY_OPEN_FILES:
+		return EMFILE;
+	case ERROR_ACCESS_DENIED:
+	case ERROR_SHARING_VIOLATION:
+		return EACCES;
+	case ERROR_NETWORK_ACCESS_DENIED:
+	case ERROR_FAIL_I24:
+	case ERROR_SEEK_ON_DEVICE:
+		return EACCES;
+	case ERROR_LOCK_VIOLATION:
+	case ERROR_DRIVE_LOCKED:
+	case ERROR_NOT_LOCKED:
+	case ERROR_LOCK_FAILED:
+		return EACCES;
+	case ERROR_INVALID_HANDLE:
+		return EBADF;
+	case ERROR_NOT_ENOUGH_MEMORY:
+	case ERROR_INVALID_BLOCK:
+	case ERROR_NOT_ENOUGH_QUOTA:
+		return ENOMEM;
+	case ERROR_INVALID_ACCESS:
+	case ERROR_INVALID_DATA:
+	case  ERROR_INVALID_PARAMETER:
+		return EINVAL;
+	case ERROR_BROKEN_PIPE:
+	case ERROR_NO_DATA:
+		return EPIPE;
+	case ERROR_DISK_FULL:
+		return ENOSPC;
+	case ERROR_ALREADY_EXISTS:
+		return EEXIST;
+	case ERROR_NESTING_NOT_ALLOWED:
+		return EAGAIN;
+	}
+
+	/* try to detect error by range */
+	if (MIN_EACCES_RANGE <= error_code && error_code <= MAX_EACCES_RANGE) {
+		return EACCES;
+	} else {
+		return EINVAL;
+	}
+}
+
+/**
+ * Assign errno to the error value converted from the GetLastError().
+ */
+void set_errno_from_last_file_error(void)
+{
+	errno = convert_last_error_to_errno();
+}
+
 /**
  * Concatenate directory path with filename, unicode version.
  *
@@ -248,7 +291,10 @@ wchar_t* make_pathw(const wchar_t* dir_path, size_t dir_len, wchar_t* filename)
 	res = (wchar_t*)rsh_malloc((dir_len + len + 2) * sizeof(wchar_t));
 	if(dir_len > 0) {
 		memcpy(res, dir_path, dir_len * sizeof(wchar_t));
-		res[dir_len++] = (wchar_t)SYS_PATH_SEPARATOR;
+		if(res[dir_len - 1] != (wchar_t)SYS_PATH_SEPARATOR) {
+			/* append path separator to the directory */
+			res[dir_len++] = (wchar_t)SYS_PATH_SEPARATOR;
+		}
 	}
 
 	/* append filename */
@@ -256,60 +302,6 @@ wchar_t* make_pathw(const wchar_t* dir_path, size_t dir_len, wchar_t* filename)
 	return res;
 }
 
-/**
- * Expand wildcards in the given filepath and store results into vector.
- * If no wildcards are found then just the filepath is stored.
- * Note: only wildcards in the last filename of the path are expanded.
- *
- * @param vect the vector to receive wide-strings with file paths
- * @param filepath the filepath to process
- */
-void expand_wildcards(vector_t* vect, wchar_t* filepath)
-{
-	int added = 0;
-	size_t len = wcslen(filepath);
-	size_t index = wcscspn(filepath, L"*?");
-
-	/* if a wildcard has been found without a directory separator after it */
-	if(index < len && wcscspn(filepath + index, L"/\\") >= (len - index))
-	{
-		wchar_t* parent;
-		WIN32_FIND_DATAW d;
-		HANDLE h;
-
-		/* find directory separator */
-		for(; index > 0 && !IS_PATH_SEPARATOR(filepath[index]); index--);
-		parent = (IS_PATH_SEPARATOR(filepath[index]) ? filepath : 0);
-
-		h = FindFirstFileW(filepath, &d);
-		if(INVALID_HANDLE_VALUE != h) {
-			do {
-				wchar_t* wpath;
-				char* cstr;
-				int failed;
-				if((0 == wcscmp(d.cFileName, L".")) || (0 == wcscmp(d.cFileName, L".."))) continue;
-				if(NULL == (wpath = make_pathw(parent, index + 1, d.cFileName))) continue;
-				cstr = wchar_to_cstr(wpath, WIN_DEFAULT_ENCODING, &failed);
-				/* note: just quietly skip unconvertible file names */
-				if(!cstr || failed) {
-					free(cstr);
-					free(wpath);
-					continue;
-				}
-				rsh_vector_add_ptr(vect, cstr);
-				added++;
-			} while(FindNextFileW(h, &d));
-			FindClose(h);
-		}
-	}
-
-	if(added == 0) {
-		wchar_t* wpath = make_pathw(0, 0, filepath);
-		char* cstr = w2c(wpath);
-		if(cstr) rsh_vector_add_ptr(vect, cstr);
-	}
-}
-
 /* functions to setup/restore console */
 
 /**
@@ -329,6 +321,8 @@ void setup_console(void)
 	{
 		rhash_data.saved_console_codepage = GetConsoleOutputCP();
 		SetConsoleOutputCP(cp);
+		setlocale(LC_CTYPE, opt.flags&OPT_UTF8 ? "C" :
+			opt.flags&OPT_ANSI ? ".ACP" : ".OCP");
 		rsh_exit = rhash_exit;
 	}
 
@@ -408,7 +402,7 @@ WIN_DIR* win_opendir(const char* dir_path)
 	d->hFind = (wpath != NULL ?
 		FindFirstFileW(wpath, &d->findFileData) : INVALID_HANDLE_VALUE);
 	free(wpath);
-	
+
 	if(d->hFind == INVALID_HANDLE_VALUE && GetLastError() != ERROR_ACCESS_DENIED) {
 		wpath = c2w(path, 1); /* try to use secondary codepage */
 		if(wpath) {
@@ -417,12 +411,13 @@ WIN_DIR* win_opendir(const char* dir_path)
 		}
 	}
 	free(path);
-	
+
 	if(d->hFind == INVALID_HANDLE_VALUE && GetLastError() == ERROR_ACCESS_DENIED) {
 		free(d);
 		errno = EACCES;
 		return NULL;
 	}
+	set_errno_from_last_file_error();
 
 	d->state = (d->hFind == INVALID_HANDLE_VALUE ? -1 : 0);
 	d->dir.d_name = NULL;
