@@ -18,8 +18,6 @@ char* win_to_utf8(const char* str);
 #define WIN_DEFAULT_ENCODING -1
 char* wchar_to_cstr(const wchar_t* wstr, int codepage, int* failed);
 
-void expand_wildcards(struct vector_t* vect, wchar_t* filepath);
-
 /* file functions */
 FILE* win_fopen_ex(const char* path, const char* mode, int exclusive);
 
@@ -27,7 +25,7 @@ FILE* win_fopen_ex(const char* path, const char* mode, int exclusive);
 #define win_fopen_bin(path, mode) win_fopen_ex(path, mode, 1)
 int win_stat(const char* path, struct rsh_stat_struct *buffer);
 int can_open_exclusive(const char* path);
-void win32_set_filesize64(const char* path, uint64_t *pSize);
+void set_errno_from_last_file_error(void);
 wchar_t* make_pathw(const wchar_t* dir_path, size_t dir_len, wchar_t* filename);
 
 void set_benchmark_cpu_affinity(void);
